#!/usr/bin/Rscript

## Name: adjust.R
## Version: 1.2.6
## Date: May 17, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

##################################################
#### load libraries
##################################################
library(getopt)

##################################################
#### get options
spec <- matrix(c(
    'input', 'i', 1, "character", "input file - gff result from RNAprof (required)",
    'adjust', 'a', 2, "character", "adjust method (optional, BH by default)",
    'output', 'o', 2, "character", "output file (optional, output.gff by default)",
    'padj', 'p', 2, "double", "threshold p-value (optional, 1e-3 by default)",
    'help', 'h', 0, "logical", "help"
    ), ncol=5, byrow=T)

##< check options
tryCatch( {
    opt = getopt(spec);
}, error=function(e) {
    cat(paste(getopt(spec, usage=T),"\n"));
    quit()
})

if (!is.null(opt$help) || is.null(opt$input)) {
    cat(paste(getopt(spec, usage=T),"\n"));
    quit();
}
##>

##< required arguments
inputFileName <- opt$input
##>

##< constant
adjustMethod <- "BH"
outputFileName <- "output.gff"
fdr <- 1e-3
##>

if (!is.null(opt$adjust)) {
    adjustMethod <- opt$adjust
}
if (!is.null(opt$output)) {
    outputFileName <- opt$output
}
if (!is.null(opt$padj)) {
    fdr <- opt$padj
}
##################################################

## read results from RNAprof
data <- read.table(inputFileName, sep="\t")
data9 <- strsplit(as.character(data[, 9]), ";")
data9.refined <- lapply(data9, FUN=function(x) {
    len <- length(x)
    x[c(1, (len-5):len)]
})
data9.matrix <- do.call(rbind, data9.refined)
data[, 9] <- apply(data9.matrix, 1, paste, collapse=";")
data.cat <- apply(data, 1, paste, collapse="\t")
data <- do.call(rbind, strsplit(data.cat, "="))

## get p-values from loci.gff
rawp <- data[, ncol(data)]

## adjust p-values using adjustMethod
adjp <- p.adjust(rawp, method=adjustMethod)

## add adjusted p-values
data[, ncol(data)] <- paste(data[, ncol(data)], ";P-adj", sep="")

## compute number of events and genes
keep.adjp <- data.frame(do.call(rbind, strsplit(as.character(data[, 2]), ":")))
keep.adjp <- cbind(keep.adjp, adjp)
keep.adjp <- keep.adjp[as.numeric(keep.adjp[, ncol(keep.adjp)]) <= fdr,]

## write to files
if (!is.na(file.info("reportadj.txt")$size)) {
    file.remove("reportadj.txt")
}
cat(paste("Number of events: ", nrow(keep.adjp), "\n", sep=""), file="reportadj.txt", append=T, sep="\t")
cat(paste("Number of genes with events: ", length(unique(keep.adjp[, 1])), "\n", sep=""), file="reportadj.txt", append=T, sep="\t")

adjp <- format(adjp, digits=2, scientific=TRUE)
data <- cbind(data, adjp)
data <- data.frame(matrix(sapply(data, gsub, pattern="\"", replacement=""), nrow=nrow(data), ncol=ncol(data)))
write.table(data, file=outputFileName, quote=FALSE, col.names=FALSE, row.names=FALSE, sep="=")
